"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Url = require('url-parse');
var beanStalk = require('nodestalker');
var Aws = require('aws-sdk');
var Fs = require('fs');

var PractoQueue = function () {
  function PractoQueue() {
    _classCallCheck(this, PractoQueue);
  }

  _createClass(PractoQueue, null, [{
    key: 'getInstance',
    value: function getInstance(uri) {
      var sqsSpoolDir = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      var parts = new Url(uri);
      parts.protocol = parts.protocol.replace(':', '');
      if (parts.protocol === 'https') {
        var host = parts.hostname;
        var hostParts = host.split('.');
        if (hostParts[0] === 'sqs' && hostParts[2] === 'amazonaws' && hostParts[3] === 'com') {
          return new Sqs(uri, sqsSpoolDir);
        } else {
          throw 'Invalid Sqs URI';
        }
      } else if (parts.protocol === 'beanstalk') {
        return new Beanstalk(uri);
      } else {
        throw 'Unsupported URI scheme';
      }
    }
  }]);

  return PractoQueue;
}();

var Queue = function () {
  function Queue() {
    _classCallCheck(this, Queue);
  }

  _createClass(Queue, [{
    key: 'getQueueName',
    value: function getQueueName(queueIdentifier) {
      var host = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      var queueName = queueIdentifier;
      if (host) {
        var baseUrl = host.split('/')[2];
        var subDomain = baseUrl.split('.')[0];
        var subDomainParts = subDomain.split('-');
        if (subDomainParts.length > 1) {
          queueName = queueName + '-' + subDomainParts[1];
        }
      }
      return queueName;
    }
  }, {
    key: 'setDefaultHost',
    value: function setDefaultHost() {
      var host = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

      this.host = host;
    }
  }]);

  return Queue;
}();

var Sqs = function (_Queue) {
  _inherits(Sqs, _Queue);

  function Sqs(uri) {
    var sqsSpoolDir = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    _classCallCheck(this, Sqs);

    var _this = _possibleConstructorReturn(this, (Sqs.__proto__ || Object.getPrototypeOf(Sqs)).call(this));

    _this.sqsSpoolDir = sqsSpoolDir;
    var parts = new Url(uri);
    var host = parts.hostname;
    var hostParts = host.split('.', 2);
    _this.region = hostParts[1];
    var awsAccessKeyId = parts.username;
    var awsSecretAccessKey = parts.password;
    Aws.config = new Aws.Config();
    Aws.config.accessKeyId = awsAccessKeyId;
    Aws.config.secretAccessKey = awsSecretAccessKey;
    Aws.config.region = _this.region;
    _this.sqs = new Aws.SQS();
    var path = parts.pathname;
    if (path.startsWith('/')) {
      path = path.substr(1);
    }
    var pathParts = path.split('/');
    _this.accountId = pathParts[0];
    parts.protocol = parts.protocol.replace(':', '');
    _this.protocol = parts.protocol;
    _this.queuePrefix = '';
    var lastPart = pathParts[1];
    if (lastPart.endsWith('_') || lastPart.endsWith('-')) {
      _this.queuePrefix = lastPart;
    }
    _this.queue = null;
    _this.queueName = '';
    _this.host = null;
    _this.BASE64_EXPAND_RATIO = 1.37;
    _this.MAX_JOB_SIZE = 262144;
    return _this;
  }

  _createClass(Sqs, [{
    key: 'setDefaultQueue',
    value: function setDefaultQueue(queueName, callback) {
      var _this2 = this;

      queueName = this.getQueueName(queueName, this.host);
      this.queueName = this.queuePrefix + queueName;
      this.sqs.getQueueUrl({ QueueName: this.queueName }, function (err, data) {
        if (err) {
          if (err.statusCode == 400) {
            var params = {
              QueueName: queueName
            };
            _this2.sqs.createQueue(params, function (err, data) {
              if (err) {
                callback(err);
              } else {
                callback(null, data.QueueUrl);
              }
            });
          } else {
            callback(err);
          }
        } else {
          callback(null, data.QueueUrl);
        }
      });
    }
  }, {
    key: 'insertJob',
    value: function insertJob(jobBody, queueName) {
      var _this3 = this;

      var delay = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      jobBody = new Buffer(jobBody).toString('base64');
      if (delay > 900) {
        delay = 900;
      }
      var sqsParams = {
        MessageBody: jobBody,
        QueueUrl: this.queue,
        DelaySeconds: delay
      };
      var queue = this.queue;
      if (this.sqsSpoolDir) {
        // LOGIC HERE FOR SPOOL
        var jobDir = '';
        queueName = this.queuePrefix + queueName;
        queueName = this.getQueueName(queueName, this.host);
        if (!Fs.existsSync(this.sqsSpoolDir)) {
          Fs.mkdirSync(this.sqsSpoolDir);
        }
        if (!jobBody) {
          throw 'Job body cannot be empty';
        }
        if (jobBody.length * this.BASE64_EXPAND_RATIO > this.MAX_JOB_SIZE) {
          throw 'One or more parameters are invalid. Reason:' + ' Message must be shorter than 262144 bytes.';
        }
        var currentTime = +new Date() / 1000;
        jobDir = parseInt(currentTime / 100).toString();
        if (!Fs.existsSync(this.sqsSpoolDir + '/' + jobDir)) {
          try {
            Fs.mkdirSync(this.sqsSpoolDir + '/' + jobDir);
          } catch (err) {
            if (err.code !== 'EEXIST') throw err;
          }
        }

        currentTime = +new Date();
        var fileName = parseInt(currentTime).toString() + this.getRandomString(10);
        var encodeBody = new Buffer(jobBody).toString('base64');
        var data = [this.region, this.getSqsQueueUrl(queueName), encodeBody, delay.toString(), '60'];
        var payload = data.join('\t');
        var filePath = this.sqsSpoolDir + '/' + jobDir + '/' + fileName;
        Fs.writeFile(filePath, payload, function (err) {
          if (err) {
            throw "Error while writing file";
          }
        });
        Fs.rename(filePath, filePath + '.v1message', function (err) {
          if (err) {
            throw 'ERROR' + err;
          }
        });
      } else {
        if (queueName !== this.queueName) {
          queueName = this.queuePrefix + queueName;
          queueName = this.getQueueName(queueName, this.host);
          this.sqs.getQueueUrl({ QueueName: queueName }, function (err, data) {
            if (err) {
              if (err.statusCode == 400) {
                var params = {
                  QueueName: queueName
                };
                _this3.sqs.createQueue(params, function (err, data) {
                  if (err) {
                    throw 'ERR: ' + err;
                  } else {
                    sqsParams.QueueUrl = data.QueueUrl;
                    _this3.sqs.sendMessage(sqsParams, function (err, data) {
                      if (err) {
                        throw 'ERR: ' + err;
                      }
                    });
                  }
                });
              } else {
                throw 'ERR: ' + err;
              }
            } else {
              sqsParams.QueueUrl = data.QueueUrl;
              _this3.sqs.sendMessage(sqsParams, function (err, data) {
                if (err) {
                  throw 'ERR: ' + err;
                }
              });
            }
          });
        } else {
          this.sqs.sendMessage(sqsParams, function (err, data) {
            if (err) {
              throw 'ERR: ' + err;
            }
          });
        }
      }
    }
  }, {
    key: 'getSqsQueueUrl',
    value: function getSqsQueueUrl(queueName) {
      return this.protocol + "://sqs." + this.region + ".amazonaws.com/" + this.accountId + "/" + queueName;
    }
  }, {
    key: 'deleteJob',
    value: function deleteJob(queueJob, queueName) {
      var queueUrl = this.getSqsQueueUrl(queueName);
      var sqsParams = {
        QueueUrl: queueUrl,
        ReceiptHandle: queueJob.ReceiptHandle
      };
      this.sqs.deleteMessage(sqsParams, function (err, data) {});
    }
  }, {
    key: 'getRandomString',
    value: function getRandomString(count) {
      var base = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-";
      var ret = "";
      var strlen = base.length;
      for (var i = 0; i < count; i++) {
        ret = ret + base[Math.floor(Math.random() * (strlen - 0) + 0)];
      }
      return ret;
    }
  }, {
    key: 'getJob',
    value: function getJob(queueName, callback) {
      var queueUrl = this.getSqsQueueUrl(queueName);
      var sqsParams = {
        QueueUrl: queueUrl,
        WaitTimeSeconds: 20
      };
      this.sqs.receiveMessage(sqsParams, function (err, data) {
        while (1) {
          if (data.Messages) {
            break;
          }
        }
        var message = data.Messages[0];
        var messageBody = message.Body;
        var messageId = message.MessageId;
        var receiptHandle = message.ReceiptHandle;
        var queueJob = new QueueJob(messageId, messageBody);
        callback(queueJob);
      });
    }
  }]);

  return Sqs;
}(Queue);

var Beanstalk = function (_Queue2) {
  _inherits(Beanstalk, _Queue2);

  function Beanstalk(uri) {
    _classCallCheck(this, Beanstalk);

    var _this4 = _possibleConstructorReturn(this, (Beanstalk.__proto__ || Object.getPrototypeOf(Beanstalk)).call(this));

    var parts = new Url(uri);
    var host = parts.hostname;
    var port = 11300;
    if (parts.port) {
      port = parts.port;
    }
    var path = parts.pathname;
    if (path.startsWith('/')) {
      path = path.substr(1);
    }
    var pathParts = path.split('/');
    _this4.queuePrefix = '';
    var lastPart = pathParts[0];
    if (lastPart.endsWith('_') || lastPart.endsWith('-')) {
      _this4.queuePrefix = lastPart;
    }
    var connectionString = host + ':' + port;
    _this4.beanStalkClient = beanStalk.Client(connectionString);
    _this4.host = null;
    return _this4;
  }

  _createClass(Beanstalk, [{
    key: 'setDefaultQueue',
    value: function setDefaultQueue(queueName) {
      queueName = this.getQueueName(queueName, this.host);
      this.queueName = this.queuePrefix + queueName;
    }
  }, {
    key: 'insertJob',
    value: function insertJob(jobBody, queueName) {
      var _this5 = this;

      var delay = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      queueName = this.getQueueName(queueName, this.host);
      this.beanStalkClient.use(this.queuePrefix + queueName).onSuccess(function (data) {
        _this5.beanStalkClient.put(jobBody).onSuccess(function (data) {
          console.log(data);
          //  return data;
        });
      });
    }
  }, {
    key: 'deleteJob',
    value: function deleteJob(queueJob) {
      this.beanStalkClient.deleteJob(queueJob.id).onSuccess(function () {});
    }
  }, {
    key: 'getJob',
    value: function getJob(callback) {
      var _this6 = this;

      this.beanStalkClient.watch(this.queueName).onSuccess(function (data) {
        _this6.beanStalkClient.reserve().onSuccess(function (job) {
          var queueJob = new QueueJob(job.id, job.data);
          callback(queueJob);
        });
      });
    }
  }]);

  return Beanstalk;
}(Queue);

var QueueJob = function () {
  function QueueJob(jobId, body) {
    _classCallCheck(this, QueueJob);

    this.jobId = jobId;
    this.body = body;
  }

  _createClass(QueueJob, [{
    key: 'getBody',
    value: function getBody() {
      return this.body;
    }
  }, {
    key: 'body',
    value: function body() {
      return this.body;
    }
  }, {
    key: 'getId',
    value: function getId() {
      return this.id;
    }
  }, {
    key: 'getReceiptHandle',
    value: function getReceiptHandle() {
      return this.receiptHandle;
    }
  }, {
    key: 'setReceiptHandle',
    value: function setReceiptHandle(receiptHandle) {
      return this.receiptHandle = receiptHandle;
    }
  }]);

  return QueueJob;
}();

module.exports = {
  PractoQueue: PractoQueue,
  Queue: Queue,
  Beanstalk: Beanstalk,
  QueueJob: QueueJob,
  Sqs: Sqs
};