"use strict"

let base =  require('./base');
let types = require('./types');

class MailGenericPromotional extends base.Base{
  constructor(host, config, appName){
    super(host, config, appName);
  }

  sendMail(fromName, fromMail, subject, body, emailType, files,
          to, hash, referenceId, referenceType, toName="",
          cc="", bcc="", plaintext="", replyTo=null, expiryTime=null,
          createdBy=null){
    this.validateMailType(emailType.toUpperCase());
    let job = new types.GenericPromotionalMail();
    job.from_name = fromName;
    job.from_address = fromMail;
    job.to_name = toName;
    job.to_address = to;
    job.expiry_time = expiryTime;
    job.subject = subject;
    job.body = body;
    job.plaintext = plaintext;
    job.reference_id = referenceId;
    job.reference_type = referenceType;
    job.app_name = this.appName;
    job.created_by = createdBy;
    job.email_type = emailType.toUpperCase();
    job.reply_to = replyTo;
    job.hash = hash;
    job.mail_type = 'generic_promotional';

    if(referenceId !== parseInt(referenceId, 10)){
      throw "referenceId should be integer";
    }
    if(typeof(cc) == 'string' && cc.length>0){
      job.cc = [cc];
    }else{
      job.cc = cc;
    }
    if(typeof(bcc) == 'string' && bcc.length>0){
      job.bcc = [bcc];
    }else{
      job.bcc = bcc;
    }

    let attachments = [];
    if(files){
      for(let i=0;i<files.length;i++){
        let att = new types.Attachment();
        att.name = files[i];
        att.location = files[files[i]];
        attachments.push(att);
      }
    }
    job.attachments = attachments;
    this.send(job, this.queueNames.sendGenericPromotionalMailQueue, 0);
  }

  getMailCountByType(mailType, startDate, endDate, deliveryStatus, callback){
    let data = {};
    data.start_date = startDate;
    data.end_date = endDate;
    data.mail_type = mailType;
    data.delivery_status = deliveryStatus;
    this.get(
      '/genericpromotionalmail/get_mail_count_for_type', data, (err, response) =>{
      if(err){
        callback(err);
      }
      else{
        callback(null, response);
      }
    });
  }

  validateMailType(type){
    const allowedTypes = [];

    if(allowedTypes.indexOf(type) < 0){
      throw "Invalid mail type";
    }
  }
}

module.exports = {
  MailGenericPromotional: MailGenericPromotional
}
