"use strict"

let basePush =  require('./basePush');
let types = require('./types');
let exceptions = require('./exceptions');
let queueNames = require('./queueNames');

let BasePush = basePush.BasePush;
let PatientTransIosPushObj = types.PatientTransIosPushObj;
let PushValidationException = exceptions.PushValidationException;


class PatientTransIosPush extends BasePush {
    constructor(host, config, appName) {
        super(host, config, appName);
        this.comm_app_name = 'fabric-ios';
        this.pushNotification = new PatientTransIosPushObj();
        this.pushNotification.service = appName;
        this.queue_name = queueNames.sendPatientTransPushQueue;
    }

    validateFilters(inputDict) {
        this.validatePushRelationKeys(inputDict);
        if ((this.isEmpty(inputDict.account_id)) && (this.isEmpty(inputDict.device_id)))
            throw new PushValidationException(
                "Account/Device ID is mandatory for transactional pushes"
            );
    }

    validatePushType(pushType) {
        let allowed_types = [
            'PHR_RECORD_SHARED_NOTIF',
            'PHR_APPOINTMENT_RECORD_UPLOAD_NOTIF',
            'PHR_ADD_REMINDER_NOTIF',
            'ACCOUNTS_REMOTE_LOGOUT',
            'TODO_SILENT_PUSH'
        ];
        if (allowed_types.indexOf(pushType) == -1)
            throw new PushValidationException("Invalid push notification type");
    }
}


module.exports = {
    PatientTransIosPush: PatientTransIosPush
}
